
//------------------------------------------------------------------------------
// Copyright (c) 2006 by AUDACIA Software
//------------------------------------------------------------------------------

package com.audacia.opentools.a86viewer;

import java.util.*;

import com.borland.primetime.editor.*;


public class A86Scanner extends AbstractScanner
{
    public A86Scanner() {
        try {
            jbInit();
        } catch (Exception ex) {
            ex.printStackTrace();
        }
    }

    protected static HashMap keywordMap;
  protected static HashMap extendedKeywordMap;
  protected static int maxKeywordLength = 0;
  protected static int maxExtendedKeywordLength = 0;

  protected static final boolean caseSensitive = true;

  protected int commentStateFlags;

  static {
    String[] keywords = getKeywords();
    keywordMap = new HashMap(keywords.length * 2 + 1);
    maxKeywordLength = initMap(keywordMap, keywords, caseSensitive);

    String[] extKeywords = getExtendedKeywords();
    extendedKeywordMap = new HashMap(extKeywords.length * 2 + 1);
    maxExtendedKeywordLength = initMap(extendedKeywordMap, extKeywords, caseSensitive);
  }

  protected int checkString(int initialState) {
    if (ch == '"') {
      ch = buf[++bp];
      while (ch != '"' && ch != '\n' && ch != EOF) {
        ch = buf[++bp];
      }
      if (ch != EOF)
        ch = buf[++bp];
      return BasicStyleMap.STRING;
    }
    else if (ch == '\'') {
      ch = buf[++bp];
      while (ch != '\'' && ch != '\n' && ch != EOF) {
        ch = buf[++bp];
      }
      if (ch != EOF)
        ch = buf[++bp];
      return BasicStyleMap.STRING;
    }
    return -1;
  }

  protected int checkNumber(int initialState) {
    if (isNumberStart (ch))
    {
      ch = buf[++bp];
      while (isNumber (ch) && ch != '\n' && ch != EOF)
        ch = buf[++bp];
      if (ch != EOF)
        ch = buf[++bp];
      return BasicStyleMap.NUMBER;
    }

    return -1;
  };


  protected int checkIdentifier(int initialState) {
    if (isValidIdentifierStart(ch) && ch != EOF) {
      int mark = bp;
      ch = buf[++bp];
      while (isValidIdentifierPart(ch) && ch != EOF)
        ch = buf[++bp];

      String text = new String(buf, mark, bp - mark);

      if (isKeyword(text))
        return BasicStyleMap.RESERVED_WORD;
      else if (isExtendedKeyword(text))
        return BasicStyleMap.EXTRA_KEYWORD;
      else
        return BasicStyleMap.IDENTIFIER;
    }
    return -1;
  }

  protected int checkComment(int initialState) {
    if (ch == ';') {
      ch = buf[++bp];
      while (ch != '\n' && ch != EOF)
        ch = buf[++bp];
      return BasicStyleMap.COMMENT;
    }

    return -1;
  }


  protected boolean isNumberStart (char ch) {
    switch (ch) {
      case 'A': case 'B': case 'C': case 'D': case 'E':
      case 'F': case 'H':
      case 'a': case 'b': case 'c': case 'd': case 'e':
      case 'f': case 'h':
      case '1': case '2': case '3': case '4': case '5':
      case '6': case '7': case '8': case '9': case '0':
        return true;
      default:
        return false;
    }
  }
  protected boolean isNumber (char ch) {
    switch (ch) {
      case '1': case '2': case '3': case '4': case '5':
      case '6': case '7': case '8': case '9': case '0':
        return true;
      default:
        return false;
    }
  }

  public static String[] getKeywords() {
    return new String[] {
      "aaa", "aad", "aam", "aas", "adc", "add", "and", "arpl", "bound", "bsf", "bsr", "bswap", "bt",
      "btc", "btr", "bts", "call", "callf", "cbw", "cdq", "clc", "cld", "cli", "clts", "cmc", "cmp", "cmps",
      "cmpsb", "cmpsd", "cmpsw", "cmpxchg", "cwd", "cwde", "daa", "das", "dec", "div", "enter",

      "f2xm1", "fabs", "fadd", "faddp", "fbld", "fbstp", "fchs", "fclex",
      "fcmovb", "fcmovbe", "fcmove", "fcmovnb", "fcmovnbe", "fcmovne", "fcmovnu",
      "fcmovu", "fcom", "fcomi", "fcomip", "fcomp", "fcompp", "fcos", "fdecstp", "fdiv",
      "fdivp", "fdivr", "fdivrp", "femms", "ffree", "fiadd", "ficom", "ficomp", "fidiv",
      "fidivr", "fild", "fimul", "fincstp", "finit", "fist", "fistp", "fisub", "fisubr",
      "fld", "fld1", "fldcw", "fldenv", "fldl2e", "fldl2t", "fldlg2", "fldln2", "fldpi",
      "fldz", "fmul", "fmulp", "fnclex", "fninit", "fnop", "fnsave", "fnstcw",
      "fnstenv", "fnstsw", "fpatan", "fprem1", "fptan", "frndint", "frstor", "fsave",
      "fscale", "fsin", "fsincos", "fsqrt", "fst", "fstcw", "fstenv", "fstp", "fstsw",
      "fsub", "fsubp", "fsubr", "fsubrp", "ftst", "fucom", "fucomi", "fucomip",
      "fucomp", "fucompp", "fwait", "fxch", "fxtract", "fyl2xp1",

      "hlt", "idiv", "imul", "in", "inc", "ins", "insb", "insd", "insw", "int", "into", "invd",
      "invlpg", "iret", "iretd", "iretw", "ja", "jae", "jb", "jbe", "jc", "jcxz", "je", "jecxz",
      "jg", "jge", "jl", "jz", "jg", "jge", "jl", "jle", "jna", "jnae", "jnb", "jnbe", "jnc", "jne",
      "jng", "jnge", "jnl", "jnle", "jno", "jnp", "jns", "jnz", "jo", "jp", "jpe", "jpo", "js", "jmp",

      "lahf", "lar", "lea", "leave", "lgdt", "lidt", "lgs", "lss", "lfs", "lds", "les", "lldt",
      "lmsw", "lock", "lods", "lodsb", "lodsd", "lodsw", "loop", "loope", "loopne",
      "loopnz", "loopz", "lsl", "ltr", "mov", "movs", "movsb", "movsw", "movsd", "movsx",
      "movzx", "mul", "neg", "nop", "not", "or", "out", "outs", "outsb", "outsd", "outsw",
      "pop", "popa", "popad", "popaw", "popf", "popfd", "popfw", "push", "pusha", "pushad",
      "pushaw", "pushf", "pushfd", "pushfw", "rcl", "rcr", "rol", "ror", "rep", "repe",
      "repne", "repnz", "repz", "ret", "retf", "sahf", "sal", "sar", "shl", "shr", "sbb", "scas",
      "scasb", "scasd", "scasw", "seta", "setae", "setb", "setbe", "setc", "sete", "setg",
      "setge", "setl", "setle", "setna", "setnae", "setnb", "setnbe", "setnc", "setne",
      "setng", "setnge", "setnl", "setnle", "setno", "setnp", "setns", "setnz", "seto",
      "setp", "setpe", "setpo", "sets", "setz", "sgdt", "sidt", "shld", "shrd", "sldt",
      "smsw", "stc", "std", "sti", "stos", "stosb", "stosd", "stosw", "str", "sub", "test",
      "verr", "verw", "wait", "wbinvd", "xadd", "xchg", "xlat", "xlatb", "xor",

      "ax", "bx", "cx", "dx", "ex", "si", "di", "bp", "sp", "ss", "es", "ds", "cs", "ip",
      "al", "ah", "bl", "bh", "ch", "cl", "dh", "dl", "eh", "el",
      "eax", "ebx", "ebp", "ecx", "edi", "edx", "esi", "esp",

      "%out", ".186", ".286", ".286c", ".286p", ".287", ".386", ".386p", ".387", ".8086", ".W", ".B", ".D",
      ".8087", ".alpha", ".seq", ".code", ".const", ".cref", ".data", ".data?", ".err",
      ".err1", ".err2", ".errb", ".errdef", ".errdif", ".erre", ".fardata",
      ".fardata?", ".lall", ".lfcond", ".list", ".model", ".msfloat", ".radix",
      ".sall", ".sfcond", ".stack", ".type", ".xall", ".xcref", ".xlist", "@curseg",
      "@filename", "@code", "@codesize", "@datasize", "@const", "@data", "@data?",
      "@fardata", "@fardata?", "@stack", "align", "and", "assume", "at", "b", "byte",
      "comm", "comment", "common", "compact", "d", "db", "dd", "df", "dosseg", "dup", "fq", "dt",
      "dw", "dword", "else", "end", "endif", "endm", "endp", "ends", "eq", "equ", "even",
      "exitm", "extrn", "far", "ge", "group", "h", "high", "huge", "ifdef", "include",
      "includelib", "irp", "irpc", "label", "large", "le", "length", "low", "local", "lt",
      "macro", "mask", "medium", "memory", "name", "near", "not", "o", "offset", "or", "org",
      "page", "para", "proc", "public", "purge", "q", "record", "rept", "seg", "segment",
      "shl", "short", "size", "shr", "small", "stack", "struc", "subttl", "this", "tiny",
      "title", "type", "use16", "use32", "width", "word", "xor", "code", "data", "nothing",
      "ptr",
    };
  }

  public static String[] getExtendedKeywords() {
    return new String[] {

    };
  }

  protected boolean isValidIdentifierStart(char ch) {
    switch (ch) {
      case 'A': case 'B': case 'C': case 'D': case 'E':
      case 'F': case 'G': case 'H': case 'I': case 'J':
      case 'K': case 'L': case 'M': case 'N': case 'O':
      case 'P': case 'Q': case 'R': case 'S': case 'T':
      case 'U': case 'V': case 'W': case 'X': case 'Y':
      case 'Z':
      case 'a': case 'b': case 'c': case 'd': case 'e':
      case 'f': case 'g': case 'h': case 'i': case 'j':
      case 'k': case 'l': case 'm': case 'n': case 'o':
      case 'p': case 'q': case 'r': case 's': case 't':
      case 'u': case 'v': case 'w': case 'x': case 'y':
      case 'z':
      case '_': case '%': case '@': case '.':
        return true;
      default:
        return Character.isJavaIdentifierStart(ch);
    }
  }

  protected boolean isSymbol(char ch) {
    switch (ch) {
      case ',': case ':':
      case '+': case '-': case '/': case '*':
      case '(': case ')':
      case '[': case ']': case '<': case '>':
      case '=':
        return true;
      case '&': case '|': case '!': case '^':
      default:
        return false;
    }
  }

  protected boolean isKeyword(String str) {
    if (str.length() > maxKeywordLength)
      return false;
    return keywordMap.get(str.toLowerCase ()) != null;
  }

  protected boolean isExtendedKeyword(String str) {
    if (str.length() > maxExtendedKeywordLength)
      return false;
    return extendedKeywordMap.get(str.toLowerCase ()) != null;
  }

    protected void jbInit() throws Exception {
    }

}
