
//------------------------------------------------------------------------------
// Copyright (c) 2006 by AUDACIA Software
//------------------------------------------------------------------------------

package com.audacia.opentools.casiopv.addinwiz;

import java.io.*;

import javax.swing.*;

import com.borland.cbuilder.node.*;
import com.borland.primetime.ide.*;
import com.borland.primetime.node.*;
import com.borland.primetime.util.*;
import com.borland.primetime.vfs.*;
import com.borland.primetime.wizard.*;


public class HeaderFileWizard extends BasicWizard {

  public static final WizardAction WIZARD_NewHeaderFile = new WizardAction (
    "Header File", 'H', "Creates a new header file",
    BrowserIcons.ICON_PROJECT,
    new ImageIcon(Images.getImage(HeaderFileWizard.class,
      "image/header.gif")),
      true, "PocketViewer") {
    // override method so wizard disabled if no project active
    public void update(Object source) {
      Browser browser = Browser.findBrowser(source);
      Project project = browser.getProjectView().getActiveProject();
      setEnabled(project != null);
    }
    protected Wizard createWizard() {
      return new HeaderFileWizard();
    }
  };


  public HeaderFileWizard() {
    super();
  }

  public WizardPage invokeWizard(WizardHost host) {
    setWizardTitle("New Header File Wizard");
    namePage = new FileWizardNamePage(true);
    addWizardPage(namePage);
    return super.invokeWizard(host);
  }


  protected static Url makeFileUrl(CBProject project,
                                   String subdirName,
                                   String fileName)
  {
    try {
      return new Url(project.getProjectPath() + "/" + subdirName + "/" +
                     fileName);
    }
    catch (InvalidUrlException ex) {
      return null;
    }
  }

  protected static FileNode createNode(CBProject project,
                                       String subdirName,
                                       String fileName)
  {
    LightweightNode[] lwn;
    Node fn = project;
    Url filePath = makeFileUrl (project, subdirName, fileName);
    FileNode newNode = project.getNode (filePath);
    String nodeToFind = "Header Files";

    lwn = project.findNodes (FolderNode.class);
    for (int i = lwn.length - 1; i >= 0; --i)
    {
      if (lwn[i].getName ().equals (nodeToFind))
        fn = lwn[i];
    }
    newNode.setParent (fn);
    return newNode;
  }

  protected String getGeneratedCode(String text) {
    int i, arrayOffset = HFileTemplateOffset;
    String[] array = HFileTemplate;
    String result = "";


    for (i = 0; i < array.length; ++i)
    {
      result = result + array[i];
      if ((i == 1) || (i == 2) || (i == 15))
        result = result + text.toUpperCase ();
      if (i == arrayOffset)
        result = result + text;
    }
    return (result);
  }

  protected void finish() throws VetoException {
    Browser browser = Browser.getActiveBrowser ();
    CBProject project = (CBProject) browser.getProjectView ().getActiveProject ();
    FileNode srcNode = createNode (project, "H",
                                   namePage.getName () + (".h"));

    try {
      Buffer buf = srcNode.getBuffer();
      buf.setContent(getGeneratedCode (namePage.getName()).getBytes());
      try {
        browser.setActiveNode(srcNode, true);
      }
      catch (Exception ex) {
      }
    }
    catch (IOException ex) {
    }

  }

  public void wizardCompleted() {
  }


  private final int HFileTemplateOffset = 6;
  private final String[] HFileTemplate = {
    "\n",
    "#ifndef _",
    "_H\n#define _",
    "_H\n",
    "\n",
    "/*\n",
    " * ",
    ".h\n *\n",
    " * <description>\n",
    " *\n",
    " */\n",
    "\n",
    "#include \"modeldef.h\"\n",
    "\n",
    "\n",
    "#endif /* _",
    "_H */\n\n"
  };


  private FileWizardNamePage namePage;
}
