
//------------------------------------------------------------------------------
// Copyright (c) 2006 by AUDACIA Software
//------------------------------------------------------------------------------

package com.audacia.opentools.casiopv.addinwiz;

import java.io.*;

import javax.swing.*;

import com.borland.primetime.ide.*;
import com.borland.primetime.node.*;
import com.borland.primetime.util.*;
import com.borland.primetime.vfs.*;
import com.borland.primetime.wizard.*;
import com.borland.cbuilder.node.*;


public class SourceFileWizard extends BasicWizard {

  public static final WizardAction WIZARD_NewSourceFile = new WizardAction (
    "Source File", 'S', "Creates a new Source file",
    BrowserIcons.ICON_PROJECT,
    new ImageIcon(Images.getImage(SourceFileWizard.class,
      "image/source.gif")),
      true, "PocketViewer") {
    // override method so wizard disabled if no project active
    public void update(Object source) {
      Browser browser = Browser.findBrowser(source);
      Project project = browser.getProjectView().getActiveProject();
      setEnabled(project != null);
    }
    protected Wizard createWizard() {
      return new SourceFileWizard();
    }
  };

  public SourceFileWizard() {
    super();
  }

  public WizardPage invokeWizard(WizardHost host) {
    setWizardTitle("New Source File Wizard");
    namePage = new FileWizardNamePage(false);
    addWizardPage(namePage);
    return super.invokeWizard(host);
  }


  protected static Url makeFileUrl(CBProject project,
                                   String subdirName,
                                   String fileName)
  {
    try {
      return new Url(project.getProjectPath() + "/" + subdirName + "/" +
                     fileName);
    }
    catch (InvalidUrlException ex) {
      return null;
    }
  }

  protected static FileNode createNode(CBProject project,
                                       String subdirName,
                                       String fileName, boolean a86file)
  {
    LightweightNode[] lwn;
    Node fn = project;
    Url filePath = makeFileUrl (project, subdirName, fileName);
    FileNode newNode = project.getNode (filePath);
    String nodeToFind = a86file ? "Assembler Files" : "Source Files";

    lwn = project.findNodes (FolderNode.class);
    for (int i = lwn.length - 1; i >= 0; --i)
    {
      if (lwn[i].getName ().equals (nodeToFind))
        fn = lwn[i];
    }
    newNode.setParent (fn);
    return newNode;
  }

  protected String getGeneratedCode(String text) {
    int i, arrayOffset1, arrayOffset2;
    String[] array;
    String result = "";

    if (namePage.getIsA86File ())
    {
      arrayOffset1 = A86FileTemplateOffset1;
      arrayOffset2 = A86FileTemplateOffset2;
      array = A86FileTemplate;
    }
    else
    {
      arrayOffset1 = CFileTemplateOffset;
      arrayOffset2 = arrayOffset1;
      array = CFileTemplate;
    }

    for (i = 0; i < array.length; ++i)
    {
      result = result + array[i];
      if ((i == arrayOffset1) || (i == arrayOffset2))
        result = result + text;
    }
    return (result);
  }

  protected void finish() throws VetoException {
    Browser browser = Browser.getActiveBrowser ();
    CBProject project = (CBProject) browser.getProjectView ().getActiveProject ();
    FileNode srcNode = createNode (project, "C",
                                   namePage.getName () + (namePage.getIsA86File () ? ".a86" : ".c"),
                                   namePage.getIsA86File ());

    try {
      Buffer buf = srcNode.getBuffer();
      buf.setContent(getGeneratedCode (namePage.getName()).getBytes());
      try {
        browser.setActiveNode(srcNode, true);
      }
      catch (Exception ex) {
      }
    }
    catch (IOException ex) {
    }

  }

  public void wizardCompleted() {
  }


  private final int CFileTemplateOffset = 2;
  private final String[] CFileTemplate = {
    "\n",
    "/*\n",
    " * ",
    ".c\n *\n",
    " * <description>\n",
    " *\n",
    " */\n",
    "\n",
    "#include \"modeldef.h\"\n",
    "\n",
    "\n",
  };

  private final int A86FileTemplateOffset1 = 1;
  private final int A86FileTemplateOffset2 = 8;
  private final String[] A86FileTemplate = {
    ";;;\n",
    ";;; ",
    ".a86\n;;;\n",
    ";;; <description>\n",
    ";;;\n",
    "\n",
    "\n",
    "\n",
    "",
    "_TEXT\tCSEG\n\n",
    "\t.186\n",
  };


  private FileWizardNamePage namePage;
}


