//---------------------------------------------------------------------------
// build.hpp
// part of PVBUILD
// (c) 2006 by Moritz Beutel, AUDACIA Software
//---------------------------------------------------------------------------

#ifndef _BUILD_H
#define _BUILD_H

#include <string>
#include <vector>

#include "ucl/file.hpp"
#include "tinyxml/tinyxml.h"

std::string getExePath (void);

struct BuildInfo
{
  std::vector <std::string> defines;   /* conditional defines */
  std::string               filename;  /* protofile name */
  std::string               progname;  /* program name */
  int                       version;   /* program version */
  bool                      library;   /* library */
  bool                      debug;     /* debug */
  bool                      asmopt;    /* ASM optimisation (PV2 only) */
  char                      memmodel;  /* memory model (PV2 only) */
  int                       tarseg;    /* target segment (PV2 only) */

  BuildInfo ()
    : version (100), library (false), debug (false), asmopt (false),
      memmodel ('S'), tarseg (80000)
  { }
};

class BuildTool /* abstract */
{
protected:
  std::string sdkpath;
  std::string exepath;
  std::vector <std::string> files;
  std::string objfiles, hdrfiles, libfiles;
  BuildInfo& bi;
  const ucl::IniFile settings;

  BuildTool (const ucl::IniFile& set, BuildInfo& _bi);
  bool findCbxFile (std::string& filename);
  bool parseXmlNode (TiXmlNode* node);

  virtual bool intWriteMakefile () = 0;
  virtual void addObjFile (const std::string& objfile) = 0;
  virtual void addHdrFile (const std::string& hdrfile) = 0;
  virtual void addLibFile (const std::string& libfile) = 0;

public:
  bool WriteMakefile ();
  virtual bool DoBuild () = 0;

  virtual ~BuildTool () {}
};



class PV2BuildTool : public BuildTool
{
protected:
  virtual bool intWriteMakefile ();
  virtual void addObjFile (const std::string& objfile);
  virtual void addHdrFile (const std::string& objfile);
  virtual void addLibFile (const std::string& libfile);
  bool invokeMakeTool (char model);

public:
  virtual bool DoBuild ();

  PV2BuildTool (const ucl::IniFile& set, BuildInfo& _bi);
  virtual ~PV2BuildTool () {}
};



class PV3BuildTool : public BuildTool
{
protected:
  unsigned int objcount;
  std::string libinc;

  virtual bool intWriteMakefile ();
  virtual void addObjFile (const std::string& objfile);
  virtual void addHdrFile (const std::string& objfile);
  virtual void addLibFile (const std::string& libfile);

public:
  virtual bool DoBuild ();

  PV3BuildTool (const ucl::IniFile& set, BuildInfo& _bi);
  virtual ~PV3BuildTool () {}
};




#endif // _BUILD_H




