
#include <iostream>
#include <cstdlib>
#include <cstdio>
#include <cstring>
#include <cctype>
#include <vector>
#include <algorithm>
#include <string>
#include <windows.h>
#pragma hdrstop


#include "tinyxml/tinyxml.h"

#include "ucl/file.hpp"
#include "build.hpp"

using namespace std;


#ifdef __BORLANDC__
  #pragma argsused
#endif



void showSyntaxMessage (void)
{
  cout << "CASIO PV SDK build tool and makefile creator" "\n\n\n"
          "Syntax:\n"
          " PVBUILD [-d][-l][-m][-a][-zx][-px][-Dxxx][-vxxx][-txxx][-nxxx] <protofile>" "\n\n"
          " -d    - debug build (default: false)" "\n"
          " -l    - create library (default: false)" "\n"
          " -m    - create makefile (default: true)" "\n"
          " -a    - ASM optimisation (default: false)" "\n"
          " -zx   - memory model (s: <=64kB, p: >64kB)" "\n"
          " -px   - PV model (2: old PVs (default), 3: PV-S1600)" "\n"
          " -Dxxx - defines" "\n"
          " -vxxx - version (default: 100)" "\n"
          " -txxx - target segment (default: 80000)" "\n"
          " -nxxx - program name (default: project file name)" "\n\n"
          "Example: pvbuild -d -m -p2 -v521 -DDEF1=val1;DEF2=val2 project.pvmak" "\n"
       << endl;
}


class MainClass
{
  friend main (int, char*[]);

private:
  BuildTool* bt;


  MainClass (void)
    : bt (NULL)
  { }
  ~MainClass (void)
  {
    if (bt)
      delete bt;
  }


  int main (int argc, char* argv[])
  {
    /* PVSDK Build Tool
     *
     * PVBUILD [-d][-l][-m][-a][-zx][-px][-Dxxx][-vxxx][-txxx][-nxxx] <protofile>
     *
     *  -d    - Debug build (default: false)
     *  -l    - Create library (default: false)
     *  -m    - Create makefile (default: true)
     *  -a    - ASM optimisation (default: false)
     *  -zx   - memory model (s - <= 64 kB (default), p - > 64 kB)
     *  -px   - PV model (2 - old PVs, 3 - PV-S1600)
     *  -Dxxx - defines
     *  -vxxx - version (default: 100)
     *  -txxx - target segment (default: 80000)
     *  -nxxx - program name
     *
     */

    ucl::IniFile ifSettings;
    bool makefile = true;
    BuildInfo bi;
    int pvmodel = 2;

    char* p;
    char* p2;


    cout << "PVBUILD 1.04 (c) 2006 by AUDACIA Software" << endl;

    if (argc == 1)
    {
      showSyntaxMessage ();
      return (0);
    }


    ifSettings.loadFromFile (getExePath () + "\\Settings.ini");


    for (int i = 1; i < argc; ++i)
    {
      if ((strlen (argv[i]) < 2) || ((argv[i][0] != '-') && (argv[i][0] != '/')))
      {  /* projectfile.pvmak */
        if (bi.filename.empty ())
          bi.filename = argv[i];
      }
      else
        switch (argv[i][1])
        {
        case 'd':
          bi.debug    = (argv[i][2]) != '-';
          break;
        case 'l':
          bi.library  = (argv[i][2]) != '-';
          break;
        case 'm':
          makefile    = (argv[i][2]) != '-';
          break;
        case 'a':
          bi.asmopt   = (argv[i][2]) != '-';
          break;
        case 'v':
          bi.version  = atoi (argv[i] + 2);
          break;
        case 't':
          bi.tarseg   = atoi (argv[i] + 2);
          break;
        case 'z':
          bi.memmodel = argv[i][2];
          break;
        case 'p':
          pvmodel     = atoi (argv[i] + 2);
          break;
        case 'D':
          p = argv[i] + 1;
          p2 = p + 1;
          while (*(++p))
          {
            if (*p == ';')
            {
              *p = '\0';
                bi.defines.push_back (p2);
              p2 = p + 1;
            }
          }
          if (*p2)
            bi.defines.push_back (p2);
          break;
        case 'n':
          bi.progname = argv[i] + 2;
          break;
        case '?':
          showSyntaxMessage ();
          return (0);
        default:
          cerr << "Error: Unknown option '" << argv[i] << '\'' << endl;
        }
    }

    switch (pvmodel)
    {
    case 2:
      bt = new PV2BuildTool (ifSettings, bi);
      break;
    case 3:
      bt = new PV3BuildTool (ifSettings, bi);
      break;
    };

    if (!bt)
    {
      cerr << "Unknown PV model '" << pvmodel << '\'' << endl;
      return (1);
    }

    if (makefile)
      if (!bt->WriteMakefile ())
        return (1);

    if (!bt->DoBuild ())
      return (1);

    return (0);
  }
};


int main (int argc, char* argv[])
{
  return (MainClass ().main (argc, argv));
}


