
//---------------------------------------------------------------------------
// ucl/binfile.hpp
// Version 1.00.00
//
// contains a class for working with binary files in memory
//
// final classes:
//   BinFile
//
//
// part of the Useful Classes Library (UCL)
// (c) 2006 by Moritz Beutel, AUDACIA Software
//
//---------------------------------------------------------------------------


#ifndef _UCL_BINFILE_HPP
#define _UCL_BINFILE_HPP

#include <vector>
#include <string>


namespace ucl
{

typedef unsigned char  byte;
typedef unsigned short word;
typedef unsigned int   dword;


class BinFile
{
protected:
	bool load_internal (const std::string& filename) const;
	mutable std::vector <byte> _data;

public:
	std::vector <byte>& data;

	BinFile (void)
	 : data (_data)
	{ }
	BinFile (const std::string& filename)
	 : data (_data)
	{ load_internal (filename); }
	BinFile (const BinFile& bf)
	 : _data (bf._data), data (_data)
	{ }

	const BinFile& operator = (const BinFile& bf)
	{ _data = bf._data; return (*this); }

	bool loadFromFile (const std::string& filename)
	{ return (load_internal (filename)); }
	bool saveToFile (const std::string& filename) const;
	bool loadFromHexFile (const std::string& filename);
	bool saveToHexFile (const std::string& filename) const;
};



} /* namespace ucl */

#endif
