
#define _CRT_SECURE_NO_DEPRECATE

#include <fstream>
#include <cstdlib>
#include <cstring>
#include <iostream>

#include "../binfile.hpp"


namespace ucl
{

size_t getFileSize (std::string filename)
{
	std::ifstream f;

	f.open (filename.c_str (), std::ios_base::binary | std::ios_base::in);
	if (!f.good () || f.eof () || !f.is_open ())
		return (0);
	f.seekg (0, std::ios_base::beg);
	std::ifstream::pos_type begin_pos = f.tellg ();
	f.seekg (0, std::ios_base::end);
	return (static_cast <size_t> (f.tellg () - begin_pos));
}

bool BinFile::load_internal (const std::string& filename) const
{
	std::ifstream file;
	size_t s;

	s = getFileSize (filename);
	_data.resize (s);
	file.open (filename.c_str (), std::ios_base::binary | std::ios_base::in);
	if (!file.good ())
		return (false);
	file.seekg (0, std::ios_base::beg);
	file.read ((char*) (&_data[0]), s);
	file.close ();

	return (true);
}

bool BinFile::saveToFile (const std::string& filename) const
{
	std::ofstream file;

	file.open (filename.c_str (), std::ios_base::binary | std::ios_base::out);
	if (!file.good ())
		return (false);
	file.write ((char*) (&_data[0]), _data.size ());
	file.close ();

	return (true);
}

static const char hexChars[] = {'0','1','2','3','4','5','6','7',
								'8','9','A','B','C','D','E','F'};
static inline char num2hex (byte num)
{ return (hexChars[num & 0x0F]); }
static inline byte hex2num (char hex)
{ return (hex - ((hex >= 'A') ? ('0' + 7) : '0')); }
static byte dhex2num (const char* hp)
{ return ((hex2num (hp[0]) << 4) | hex2num (hp[1])); }


static bool validateChecksum (const char* text)
{
	int i, n = (dhex2num (text + 1) + 4) * 2;
	char c = 0;

	for (i = 0; i < n; i += 2)
		c += dhex2num (text + i + 1);
	return (static_cast <char> (c + dhex2num (text + n + 1)) == 0);
}

static void setChecksum (char* text, unsigned int n)
{
	int c = 0;

	n += 4; /* signature */
	n <<= 1;
	for (int i = n - 2 + 1; i > 0; i -= 2)
		c += dhex2num (text + i);
	c = -c;
	text[n + 1] = num2hex ((c >> 4) & 0x0F);
	text[n + 2] = num2hex (c & 0x0F);
}

bool BinFile::loadFromHexFile (const std::string& filename)
{
	std::ifstream file;
	char text[80];
	bool exec = true;
	int n, i = 0;
	char* ptr;


	_data.reserve ((getFileSize (filename) * 16) / 45);
	_data.clear ();
	file.open (filename.c_str (), std::ios_base::in);
	if (!file.good ())
		return (false);

	while (exec)
	{
		file.getline (text, 80);
		if (!validateChecksum (text))
			break;

		switch (hex2num (text[8]))
		{
		case 0: /* Data record */
			data.resize (i + 16);
			ptr = text + 9;
			for (n = i + 16; i < n; ++i)
			{
				_data[i] = dhex2num (ptr);
				ptr += 2;
			}
			break;
		case 1: /* End of file record */
			exec = false;
			break;
		case 2: /* Extended segment address record */
		case 3: /* Start segment address record */
		case 4: /* Extended linear address record */
		case 5: /* Start linear address record */
			/* ignore */
			break;
		}
	}


	return (true);
}

static inline size_t calculateLineSize (int n)
{
	return (2 * (4 /* prefix */
			   + n /* data */
			   + 1 /* checksum */)
		  + 1 /* \n */
		  + 1 /* : */);
}

bool BinFile::saveToHexFile (const std::string& filename) const
{
	dword i, o, b, cnt;
	size_t size = _data.size ();
	std::ofstream file;
	char text[80];


	file.open (filename.c_str (), std::ios_base::out);
	if (!file.good ())
		return (false);

	for (i = 0; i < size; i += 16)
	{
		if ((i & 0x0000FFFF) == 0)
		{
			std::sprintf (text, ":02000002%04X__\n",
						  static_cast <word> (i >> 4) & 0xF000);
			setChecksum (text, 2);
			file.write (text, calculateLineSize (2));
		}
		cnt = 0;
		std::sprintf (text, ":__%04X00", i & 0x0000FFFF);
		for (o = 0; o < 16; ++o)
		{
			if ((i + o) == size)
				break;
			b = _data[i + o];
			text[o * 2 + 9] = num2hex (b >> 4);
			text[o * 2 + 10] = num2hex (b & 0x0F);
			++cnt;
		}

		text[1] = num2hex (cnt >> 4); text[2] = num2hex (cnt);
		text[1 + 8 + 32] = '\0';
		std::strcat (text, "__\n");
		setChecksum (text, cnt);
		file.write (text, calculateLineSize (cnt));
	}
	 /* end of file - add eof record */
	file.write (":00000001FF\n", calculateLineSize (0));

	file.close ();
	return (true);
}

} /* namespace ucl */
